# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import os
import mock

from botocore.session import Session
from tests import BaseEnvVar, temporary_file


class TestCredentialPrecedence(BaseEnvVar):

    def setUp(self):
        super(TestCredentialPrecedence, self).setUp()

        # Set the config file to something that doesn't exist so
        # that we don't accidentally load a config.
        os.environ['AWS_CONFIG_FILE'] = '~/.aws/config-missing'

    def create_session(self, *args, **kwargs):
        """
        Create a new session with the given arguments. Additionally,
        this method will set the credentials file to the test credentials
        used by the following test cases.
        """
        kwargs['session_vars'] = {
            'credentials_file': (
                None, None,
                os.path.join(os.path.dirname(__file__), 'test-credentials'),
                None)
        }

        return Session(*args, **kwargs)

    def test_access_secret_vs_profile_env(self):
        # If all three are given, then the access/secret keys should
        # take precedence.
        os.environ['AWS_ACCESS_KEY_ID'] = 'env'
        os.environ['AWS_SECRET_ACCESS_KEY'] = 'env-secret'
        os.environ['AWS_DEFAULT_PROFILE'] = 'test'

        s = self.create_session()
        credentials = s.get_credentials()

        self.assertEqual(credentials.access_key, 'env')
        self.assertEqual(credentials.secret_key, 'env-secret')

    @mock.patch('botocore.credentials.Credentials')
    def test_access_secret_vs_profile_code(self, credentials_cls):
        # If all three are given, then the access/secret keys should
        # take precedence.
        s = self.create_session(profile='test')

        client = s.create_client('s3', aws_access_key_id='code',
                                 aws_secret_access_key='code-secret')

        credentials_cls.assert_called_with(
            access_key='code', secret_key='code-secret', token=mock.ANY)

    def test_profile_env_vs_code(self):
        # If the profile is set both by the env var and by code,
        # then the one set by code should take precedence.
        os.environ['AWS_DEFAULT_PROFILE'] = 'test'
        s = self.create_session(profile='default')

        credentials = s.get_credentials()

        self.assertEqual(credentials.access_key, 'default')
        self.assertEqual(credentials.secret_key, 'default-secret')

    @mock.patch('botocore.credentials.Credentials')
    def test_access_secret_env_vs_code(self, credentials_cls):
        # If the access/secret keys are set both as env vars and via
        # code, then those set by code should take precedence.
        os.environ['AWS_ACCESS_KEY_ID'] = 'env'
        os.environ['AWS_SECRET_ACCESS_KEY'] = 'secret'
        s = self.create_session()

        client = s.create_client('s3', aws_access_key_id='code',
                                 aws_secret_access_key='code-secret')

        credentials_cls.assert_called_with(
            access_key='code', secret_key='code-secret', token=mock.ANY)

    def test_access_secret_env_vs_profile_code(self):
        # If access/secret keys are set in the environment, but then a
        # specific profile is passed via code, then the access/secret
        # keys defined in that profile should take precedence over
        # the environment variables. Example:
        #
        # ``aws --profile dev s3 ls``
        #
        os.environ['AWS_ACCESS_KEY_ID'] = 'env'
        os.environ['AWS_SECRET_ACCESS_KEY'] = 'env-secret'
        s = self.create_session(profile='test')

        credentials = s.get_credentials()

        self.assertEqual(credentials.access_key, 'test')
        self.assertEqual(credentials.secret_key, 'test-secret')

    def test_honors_aws_shared_credentials_file_env_var(self):
        with temporary_file('w') as f:
            f.write('[default]\n'
                    'aws_access_key_id=custom1\n'
                    'aws_secret_access_key=custom2\n')
            f.flush()
            os.environ['AWS_SHARED_CREDENTIALS_FILE'] = f.name
            s = Session()
            credentials = s.get_credentials()

            self.assertEqual(credentials.access_key, 'custom1')
            self.assertEqual(credentials.secret_key, 'custom2')



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from tests.functional.docs import BaseDocsFunctionalTest


class TestLexDocs(BaseDocsFunctionalTest):
    TYPE_STRING = '{...}|[...]|123|123.4|\'string\'|True|None'

    def test_jsonheader_docs(self):
        docs = self.get_docstring_for_method('lex-runtime', 'post_content')
        self.assert_contains_lines_in_order([
            '**Request Syntax**',
            'sessionAttributes=%s,' % self.TYPE_STRING,
            ':type sessionAttributes: JSON serializable',
            '**Response Syntax**',
            '\'slots\': %s,' % self.TYPE_STRING,
            '\'sessionAttributes\': %s' % self.TYPE_STRING,
            '**slots** (JSON serializable)',
            '**sessionAttributes** (JSON serializable)'
        ], docs)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore.docs.method import document_model_driven_method
from botocore.docs.waiter import document_wait_method
from botocore.docs.paginator import document_paginate_method
from botocore.docs.bcdoc.restdoc import DocumentStructure


class LazyLoadedDocstring(str):
    """Used for lazily loading docstrings

    You can instantiate this class and assign it to a __doc__ value.
    The docstring will not be generated till accessed via __doc__ or
    help(). Note that all docstring classes **must** subclass from
    this class. It cannot be used directly as a docstring.
    """
    def __init__(self, *args, **kwargs):
        """
        The args and kwargs are the same as the underlying document
        generation function. These just get proxied to the underlying
        function.
        """
        super(LazyLoadedDocstring, self).__init__()
        self._gen_args = args
        self._gen_kwargs = kwargs
        self._docstring = None

    def __new__(cls, *args, **kwargs):
        # Needed in order to sub class from str with args and kwargs
        return super(LazyLoadedDocstring, cls).__new__(cls)

    def _write_docstring(self, *args, **kwargs):
        raise NotImplementedError(
            '_write_docstring is not implemented. Please subclass from '
            'this class and provide your own _write_docstring method'
        )

    def expandtabs(self, tabsize=8):
        """Expands tabs to spaces

        So this is a big hack in order to get lazy loaded docstring work
        for the ``help()``. In the ``help()`` function, ``pydoc`` and
        ``inspect`` are used. At some point the ``inspect.cleandoc``
        method is called. To clean the docs ``expandtabs`` is called
        and that is where we override the method to generate and return the
        docstrings.
        """
        if self._docstring is None:
            self._generate()
        return self._docstring.expandtabs(tabsize)

    def __str__(self):
        return self._generate()

    # __doc__ of target will use either __repr__ or __str__ of this class.
    __repr__ = __str__

    def _generate(self):
        # Generate the docstring if it is not already cached.
        if self._docstring is None:
            self._docstring = self._create_docstring()
        return self._docstring

    def _create_docstring(self):
        docstring_structure = DocumentStructure('docstring', target='html')
        # Call the document method function with the args and kwargs
        # passed to the class.
        self._write_docstring(
            docstring_structure, *self._gen_args,
            **self._gen_kwargs)
        return docstring_structure.flush_structure().decode('utf-8')


class ClientMethodDocstring(LazyLoadedDocstring):
    def _write_docstring(self, *args, **kwargs):
        document_model_driven_method(*args, **kwargs)


class WaiterDocstring(LazyLoadedDocstring):
    def _write_docstring(self, *args, **kwargs):
        document_wait_method(*args, **kwargs)


class PaginatorDocstring(LazyLoadedDocstring):
    def _write_docstring(self, *args, **kwargs):
        document_paginate_method(*args, **kwargs)



# Copyright (c) 2012-2013 Mitch Garnaat http://garnaat.org/
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
from tests import unittest
from botocore.compat import six
from botocore.docs.bcdoc.style import ReSTStyle
from botocore.docs.bcdoc.restdoc import ReSTDocument


class TestStyle(unittest.TestCase):

    def test_spaces(self):
        style = ReSTStyle(None, 4)
        self.assertEqual(style.spaces(), '')
        style.indent()
        self.assertEqual(style.spaces(), '    ')
        style.indent()
        self.assertEqual(style.spaces(), '        ')
        style.dedent()
        self.assertEqual(style.spaces(), '    ')
        style.dedent()
        self.assertEqual(style.spaces(), '')
        style.dedent()
        self.assertEqual(style.spaces(), '')

    def test_bold(self):
        style = ReSTStyle(ReSTDocument())
        style.bold('foobar')
        self.assertEqual(style.doc.getvalue(), six.b('**foobar** '))

    def test_italics(self):
        style = ReSTStyle(ReSTDocument())
        style.italics('foobar')
        self.assertEqual(style.doc.getvalue(), six.b('*foobar* '))

    def test_p(self):
        style = ReSTStyle(ReSTDocument())
        style.start_p()
        style.doc.write('foo')
        style.end_p()
        self.assertEqual(style.doc.getvalue(), six.b('\n\nfoo\n\n'))

    def test_code(self):
        style = ReSTStyle(ReSTDocument())
        style.code('foobar')
        self.assertEqual(style.doc.getvalue(), six.b('``foobar`` '))

    def test_h1(self):
        style = ReSTStyle(ReSTDocument())
        style.h1('foobar fiebaz')
        self.assertEqual(
            style.doc.getvalue(),
            six.b('\n\n*************\nfoobar fiebaz\n*************\n\n'))

    def test_h2(self):
        style = ReSTStyle(ReSTDocument())
        style.h2('foobar fiebaz')
        self.assertEqual(
            style.doc.getvalue(),
            six.b('\n\n=============\nfoobar fiebaz\n=============\n\n'))

    def test_h3(self):
        style = ReSTStyle(ReSTDocument())
        style.h3('foobar fiebaz')
        self.assertEqual(
            style.doc.getvalue(),
            six.b('\n\n-------------\nfoobar fiebaz\n-------------\n\n'))

    def test_ref(self):
        style = ReSTStyle(ReSTDocument())
        style.ref('foobar', 'http://foo.bar.com')
        self.assertEqual(style.doc.getvalue(),
                         six.b(':doc:`foobar <http://foo.bar.com>`'))

    def test_examples(self):
        style = ReSTStyle(ReSTDocument())
        self.assertTrue(style.doc.keep_data)
        style.start_examples()
        self.assertFalse(style.doc.keep_data)
        style.end_examples()
        self.assertTrue(style.doc.keep_data)

    def test_codeblock(self):
        style = ReSTStyle(ReSTDocument())
        style.codeblock('foobar')
        self.assertEqual(style.doc.getvalue(),
                         six.b('::\n\n  foobar\n\n\n'))

    def test_important(self):
        style = ReSTStyle(ReSTDocument())
        style.start_important()
        style.end_important()
        self.assertEqual(style.doc.getvalue(),
                         six.b('\n\n.. warning::\n\n  \n\n'))

    def test_note(self):
        style = ReSTStyle(ReSTDocument())
        style.start_note()
        style.end_note()
        self.assertEqual(style.doc.getvalue(),
                         six.b('\n\n.. note::\n\n  \n\n'))

    def test_danger(self):
        style = ReSTStyle(ReSTDocument())
        style.start_danger()
        style.end_danger()
        self.assertEqual(style.doc.getvalue(),
                         six.b('\n\n.. danger::\n\n  \n\n'))

    def test_toctree_html(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'html'
        style.toctree()
        style.tocitem('foo')
        style.tocitem('bar')
        self.assertEqual(
            style.doc.getvalue(),
            six.b('\n.. toctree::\n  :maxdepth: 1'
                  '\n  :titlesonly:\n\n  foo\n  bar\n'))

    def test_toctree_man(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'man'
        style.toctree()
        style.tocitem('foo')
        style.tocitem('bar')
        self.assertEqual(style.doc.getvalue(),
                         six.b('\n\n\n* foo\n\n\n* bar\n\n'))

    def test_hidden_toctree_html(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'html'
        style.hidden_toctree()
        style.hidden_tocitem('foo')
        style.hidden_tocitem('bar')
        self.assertEqual(
            style.doc.getvalue(),
            six.b('\n.. toctree::\n  :maxdepth: 1'
                  '\n  :hidden:\n\n  foo\n  bar\n'))

    def test_hidden_toctree_non_html(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'man'
        style.hidden_toctree()
        style.hidden_tocitem('foo')
        style.hidden_tocitem('bar')
        self.assertEqual(
            style.doc.getvalue(),
            six.b(''))

    def test_href_link(self):
        style = ReSTStyle(ReSTDocument())
        style.start_a(attrs=[('href', 'http://example.org')])
        style.doc.write('example')
        style.end_a()
        self.assertEqual(
            style.doc.getvalue(),
            six.b('`example <http://example.org>`__ ')
        )

    def test_escape_href_link(self):
        style = ReSTStyle(ReSTDocument())
        style.start_a(attrs=[('href', 'http://example.org')])
        style.doc.write('foo: the next bar')
        style.end_a()
        self.assertEqual(
            style.doc.getvalue(),
            six.b('`foo\\: the next bar <http://example.org>`__ '))

    def test_handle_no_text_hrefs(self):
        style = ReSTStyle(ReSTDocument())
        style.start_a(attrs=[('href', 'http://example.org')])
        style.end_a()
        self.assertEqual(style.doc.getvalue(),
                         six.b('`<http://example.org>`__ '))

    def test_sphinx_reference_label_html(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'html'
        style.sphinx_reference_label('foo', 'bar')
        self.assertEqual(style.doc.getvalue(), six.b(':ref:`bar <foo>`'))

    def test_sphinx_reference_label_html_no_text(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'html'
        style.sphinx_reference_label('foo')
        self.assertEqual(style.doc.getvalue(), six.b(':ref:`foo <foo>`'))

    def test_sphinx_reference_label_non_html(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'man'
        style.sphinx_reference_label('foo', 'bar')
        self.assertEqual(style.doc.getvalue(), six.b('bar'))

    def test_sphinx_reference_label_non_html_no_text(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'man'
        style.sphinx_reference_label('foo')
        self.assertEqual(style.doc.getvalue(), six.b('foo'))

    def test_table_of_contents(self):
        style = ReSTStyle(ReSTDocument())
        style.table_of_contents()
        self.assertEqual(style.doc.getvalue(), six.b('.. contents:: '))

    def test_table_of_contents_with_title(self):
        style = ReSTStyle(ReSTDocument())
        style.table_of_contents(title='Foo')
        self.assertEqual(style.doc.getvalue(), six.b('.. contents:: Foo\n'))

    def test_table_of_contents_with_title_and_depth(self):
        style = ReSTStyle(ReSTDocument())
        style.table_of_contents(title='Foo', depth=2)
        self.assertEqual(style.doc.getvalue(),
                         six.b('.. contents:: Foo\n   :depth: 2\n'))

    def test_sphinx_py_class(self):
        style = ReSTStyle(ReSTDocument())
        style.start_sphinx_py_class('FooClass')
        style.end_sphinx_py_class()
        self.assertEqual(style.doc.getvalue(),
                         six.b('\n\n.. py:class:: FooClass\n\n  \n\n'))

    def test_sphinx_py_method(self):
        style = ReSTStyle(ReSTDocument())
        style.start_sphinx_py_method('method')
        style.end_sphinx_py_method()
        self.assertEqual(style.doc.getvalue(),
                         six.b('\n\n.. py:method:: method\n\n  \n\n'))

    def test_sphinx_py_method_with_params(self):
        style = ReSTStyle(ReSTDocument())
        style.start_sphinx_py_method('method', 'foo=None')
        style.end_sphinx_py_method()
        self.assertEqual(
            style.doc.getvalue(),
            six.b('\n\n.. py:method:: method(foo=None)\n\n  \n\n'))

    def test_sphinx_py_attr(self):
        style = ReSTStyle(ReSTDocument())
        style.start_sphinx_py_attr('Foo')
        style.end_sphinx_py_attr()
        self.assertEqual(style.doc.getvalue(),
                         six.b('\n\n.. py:attribute:: Foo\n\n  \n\n'))

    def test_write_py_doc_string(self):
        style = ReSTStyle(ReSTDocument())
        docstring = (
            'This describes a function\n'
            ':param foo: Describes foo\n'
            'returns: None'
        )
        style.write_py_doc_string(docstring)
        self.assertEqual(style.doc.getvalue(), six.b(docstring + '\n'))

    def test_new_line(self):
        style = ReSTStyle(ReSTDocument())
        style.new_line()
        self.assertEqual(style.doc.getvalue(), six.b('\n'))

        style.do_p = False
        style.new_line()
        self.assertEqual(style.doc.getvalue(), six.b('\n\n'))

    def test_list(self):
        style = ReSTStyle(ReSTDocument())
        style.li('foo')
        self.assertEqual(style.doc.getvalue(), six.b('\n* foo\n\n'))

    def test_non_top_level_lists_are_indented(self):
        style = ReSTStyle(ReSTDocument())

        # Start the top level list
        style.start_ul()

        # Write one list element
        style.start_li()
        style.doc.handle_data('foo')
        style.end_li()

        self.assertEqual(style.doc.getvalue(), six.b("\n\n\n* foo\n"))

        # Start the nested list
        style.start_ul()

        # Write an element to the nested list
        style.start_li()
        style.doc.handle_data('bar')
        style.end_li()

        self.assertEqual(style.doc.getvalue(),
                         six.b("\n\n\n* foo\n\n\n  \n  * bar\n  "))

    def test_external_link(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'html'
        style.external_link('MyLink', 'http://example.com/foo')
        self.assertEqual(style.doc.getvalue(),
                         six.b('`MyLink <http://example.com/foo>`_'))


    def test_external_link_in_man_page(self):
        style = ReSTStyle(ReSTDocument())
        style.doc.target = 'man'
        style.external_link('MyLink', 'http://example.com/foo')
        self.assertEqual(style.doc.getvalue(), six.b('MyLink'))



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from tests import unittest, IntegerRefresher
import threading
import math
import time


class TestCredentialRefreshRaces(unittest.TestCase):
    def assert_consistent_credentials_seen(self, creds, func):
        collected = []
        threads = []
        for _ in range(20):
            threads.append(threading.Thread(target=func, args=(collected,)))
        start = time.time()
        for thread in threads:
            thread.start()
        for thread in threads:
            thread.join()
        for creds in collected:
            # During testing, the refresher uses it's current
            # refresh count as the values for the access, secret, and
            # token value.  This means that at any given point in time,
            # the credentials should be something like:
            #
            # ReadOnlyCredentials('1', '1', '1')
            # ReadOnlyCredentials('2', '2', '2')
            # ...
            # ReadOnlyCredentials('30', '30', '30')
            #
            # This makes it really easy to verify we see a consistent
            # set of credentials from the same time period.  We just
            # check if all the credential values are the same.  If
            # we ever see something like:
            #
            # ReadOnlyCredentials('1', '2', '1')
            #
            # We fail.  This is because we're using the access_key
            # from the first refresh ('1'), the secret key from
            # the second refresh ('2'), and the token from the
            # first refresh ('1').
            self.assertTrue(creds[0] == creds[1] == creds[2], creds)

    def test_has_no_race_conditions(self):
        creds = IntegerRefresher(
            creds_last_for=2,
            advisory_refresh=1,
            mandatory_refresh=0
        )
        def _run_in_thread(collected):
            for _ in range(4000):
                frozen = creds.get_frozen_credentials()
                collected.append((frozen.access_key,
                                  frozen.secret_key,
                                  frozen.token))
        start = time.time()
        self.assert_consistent_credentials_seen(creds, _run_in_thread)
        end = time.time()
        # creds_last_for = 2 seconds (from above)
        # So, for example, if execution time took 6.1 seconds, then
        # we should see a maximum number of refreshes being (6 / 2.0) + 1 = 4
        max_calls_allowed = math.ceil((end - start) / 2.0) + 1
        self.assertTrue(creds.refresh_counter <= max_calls_allowed,
                        "Too many cred refreshes, max: %s, actual: %s, "
                        "time_delta: %.4f" % (max_calls_allowed,
                                              creds.refresh_counter,
                                              (end - start)))

    def test_no_race_for_immediate_advisory_expiration(self):
        creds = IntegerRefresher(
            creds_last_for=1,
            advisory_refresh=1,
            mandatory_refresh=0
        )
        def _run_in_thread(collected):
            for _ in range(100):
                frozen = creds.get_frozen_credentials()
                collected.append((frozen.access_key,
                                  frozen.secret_key,
                                  frozen.token))
        self.assert_consistent_credentials_seen(creds, _run_in_thread)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from tests import unittest

from botocore import regions
from botocore.exceptions import NoRegionError


class TestEndpointResolver(unittest.TestCase):
    def _template(self):
        return {
            'partitions': [
                {
                    'partition': 'aws',
                    'dnsSuffix': 'amazonaws.com',
                    'regionRegex': r'^(us|eu)\-\w+$',
                    'defaults': {
                        'hostname': '{service}.{region}.{dnsSuffix}'
                    },
                    'regions': {
                        'us-foo': {'regionName': 'a'},
                        'us-bar': {'regionName': 'b'},
                        'eu-baz': {'regionName': 'd'}
                    },
                    'services': {
                        'ec2': {
                            'endpoints': {
                                'us-foo': {},
                                'us-bar': {},
                                'eu-baz': {},
                                'd': {}
                            }
                        },
                        's3': {
                            'defaults': {
                                'sslCommonName': \
                                    '{service}.{region}.{dnsSuffix}'
                            },
                            'endpoints': {
                                'us-foo': {
                                    'sslCommonName': \
                                        '{region}.{service}.{dnsSuffix}'
                                },
                                'us-bar': {},
                                'eu-baz': {'hostname': 'foo'}
                            }
                        },
                        'not-regionalized': {
                            'isRegionalized': False,
                            'partitionEndpoint': 'aws',
                            'endpoints': {
                                'aws': {'hostname': 'not-regionalized'},
                                'us-foo': {},
                                'eu-baz': {}
                            }
                        },
                        'non-partition': {
                            'partitionEndpoint': 'aws',
                            'endpoints': {
                                'aws': {'hostname': 'host'},
                                'us-foo': {}
                            }
                        },
                        'merge': {
                            'defaults': {
                                'signatureVersions': ['v2'],
                                'protocols': ['http']
                            },
                            'endpoints': {
                                'us-foo': {'signatureVersions': ['v4']},
                                'us-bar': {'protocols': ['https']}
                            }
                        }
                    }
                },
                {
                    'partition': 'foo',
                    'dnsSuffix': 'foo.com',
                    'regionRegex': r'^(foo)\-\w+$',
                    'defaults': {
                        'hostname': '{service}.{region}.{dnsSuffix}',
                        'protocols': ['http'],
                        'foo': 'bar'
                    },
                    'regions': {
                        'foo-1': {'regionName': '1'},
                        'foo-2': {'regionName': '2'},
                        'foo-3': {'regionName': '3'}
                    },
                    'services': {
                        'ec2': {
                            'endpoints': {
                                'foo-1': {
                                    'foo': 'baz'
                                },
                                'foo-2': {},
                                'foo-3': {}
                            }
                        }
                    }
                }
            ]
        }

    def test_ensures_region_is_not_none(self):
        with self.assertRaises(NoRegionError):
            resolver = regions.EndpointResolver(self._template())
            resolver.construct_endpoint('foo', None)

    def test_ensures_required_keys_present(self):
        with self.assertRaises(ValueError):
            regions.EndpointResolver({})

    def test_returns_empty_list_when_listing_for_different_partition(self):
        resolver = regions.EndpointResolver(self._template())
        self.assertEqual([], resolver.get_available_endpoints('ec2', 'bar'))

    def test_returns_empty_list_when_no_service_found(self):
        resolver = regions.EndpointResolver(self._template())
        self.assertEqual([], resolver.get_available_endpoints('what?'))

    def test_gets_endpoint_names(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.get_available_endpoints(
            'ec2', allow_non_regional=True)
        self.assertEqual(['d', 'eu-baz', 'us-bar', 'us-foo'], sorted(result))

    def test_gets_endpoint_names_for_partition(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.get_available_endpoints(
            'ec2', allow_non_regional=True, partition_name='foo')
        self.assertEqual(['foo-1', 'foo-2', 'foo-3'], sorted(result))

    def test_list_regional_endpoints_only(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.get_available_endpoints(
            'ec2', allow_non_regional=False)
        self.assertEqual(['eu-baz', 'us-bar', 'us-foo'], sorted(result))

    def test_returns_none_when_no_match(self):
        resolver = regions.EndpointResolver(self._template())
        self.assertIsNone(resolver.construct_endpoint('foo', 'baz'))

    def test_constructs_regionalized_endpoints_for_exact_matches(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('not-regionalized', 'eu-baz')
        self.assertEqual('not-regionalized.eu-baz.amazonaws.com',
                          result['hostname'])
        self.assertEqual('aws', result['partition'])
        self.assertEqual('eu-baz', result['endpointName'])

    def test_constructs_partition_endpoints_for_real_partition_region(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('not-regionalized', 'us-bar')
        self.assertEqual('not-regionalized', result['hostname'])
        self.assertEqual('aws', result['partition'])
        self.assertEqual('aws', result['endpointName'])

    def test_constructs_partition_endpoints_for_regex_match(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('not-regionalized', 'us-abc')
        self.assertEqual('not-regionalized', result['hostname'])

    def test_constructs_endpoints_for_regionalized_regex_match(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('s3', 'us-abc')
        self.assertEqual('s3.us-abc.amazonaws.com', result['hostname'])

    def test_constructs_endpoints_for_unknown_service_but_known_region(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('unknown', 'us-foo')
        self.assertEqual('unknown.us-foo.amazonaws.com', result['hostname'])

    def test_merges_service_keys(self):
        resolver = regions.EndpointResolver(self._template())
        us_foo = resolver.construct_endpoint('merge', 'us-foo')
        us_bar = resolver.construct_endpoint('merge', 'us-bar')
        self.assertEqual(['http'], us_foo['protocols'])
        self.assertEqual(['v4'], us_foo['signatureVersions'])
        self.assertEqual(['https'], us_bar['protocols'])
        self.assertEqual(['v2'], us_bar['signatureVersions'])

    def test_merges_partition_default_keys_with_no_overwrite(self):
        resolver = regions.EndpointResolver(self._template())
        resolved = resolver.construct_endpoint('ec2', 'foo-1')
        self.assertEqual('baz', resolved['foo'])
        self.assertEqual(['http'], resolved['protocols'])

    def test_merges_partition_default_keys_with_overwrite(self):
        resolver = regions.EndpointResolver(self._template())
        resolved = resolver.construct_endpoint('ec2', 'foo-2')
        self.assertEqual('bar', resolved['foo'])
        self.assertEqual(['http'], resolved['protocols'])

    def test_gives_hostname_and_common_name_unaltered(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('s3', 'eu-baz')
        self.assertEqual('s3.eu-baz.amazonaws.com', result['sslCommonName'])
        self.assertEqual('foo', result['hostname'])

    def tests_uses_partition_endpoint_when_no_region_provided(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('not-regionalized')
        self.assertEqual('not-regionalized', result['hostname'])
        self.assertEqual('aws', result['endpointName'])

    def test_returns_dns_suffix_if_available(self):
        resolver = regions.EndpointResolver(self._template())
        result = resolver.construct_endpoint('not-regionalized')
        self.assertEqual(result['dnsSuffix'], 'amazonaws.com')



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import json
from nose.tools import assert_equal
from botocore.session import get_session


# Several services have names that don't match for one reason or another.
SERVICE_RENAMES = {
    'application-autoscaling': 'autoscaling',
    'appstream': 'appstream2',
    'dynamodbstreams': 'streams.dynamodb',
    'cloudwatch': 'monitoring',
    'efs': 'elasticfilesystem',
    'elb': 'elasticloadbalancing',
    'elbv2': 'elasticloadbalancing',
    'emr': 'elasticmapreduce',
    'iot-data': 'data.iot',
    'meteringmarketplace': 'metering.marketplace',
    'opsworkscm': 'opsworks-cm',
    'ses': 'email',
    'stepfunctions': 'states',
    'lex-runtime': 'runtime.lex',
    'mturk': 'mturk-requester',
    'resourcegroupstaggingapi': 'tagging',
    'lex-models': 'models.lex',
    'marketplace-entitlement': 'entitlement.marketplace',
}

BLACKLIST = [
    'mobileanalytics',
]


def test_endpoint_matches_service():
    backwards_renames = dict((v, k) for k, v in SERVICE_RENAMES.items())
    session = get_session()
    loader = session.get_component('data_loader')
    expected_services = set(loader.list_available_services('service-2'))

    pdir = os.path.dirname
    endpoints_path = os.path.join(pdir(pdir(pdir(__file__))),
                                  'botocore', 'data', 'endpoints.json')
    with open(endpoints_path, 'r') as f:
        data = json.loads(f.read())
    for partition in data['partitions']:
        for service in partition['services'].keys():
            service = backwards_renames.get(service, service)
            if service not in BLACKLIST:
                yield _assert_endpoint_is_service, service, expected_services


def _assert_endpoint_is_service(service, expected_services):
    assert service in expected_services


def test_service_name_matches_endpoint_prefix():
    # Generates tests for each service to verify that the endpoint prefix
    # matches the service name unless there is an explicit exception.
    session = get_session()
    loader = session.get_component('data_loader')

    # Load the list of available services. The names here represent what
    # will become the client names.
    services = loader.list_available_services('service-2')

    for service in services:
        yield _assert_service_name_matches_endpoint_prefix, loader, service


def _assert_service_name_matches_endpoint_prefix(loader, service_name):
    # Load the service model and grab its endpoint prefix
    service_model = loader.load_service_model(service_name, 'service-2')
    endpoint_prefix = service_model['metadata']['endpointPrefix']

    # Handle known exceptions where we have renamed the service directory
    # for one reason or another.
    expected_endpoint_prefix = SERVICE_RENAMES.get(service_name, service_name)
    assert_equal(
        endpoint_prefix, expected_endpoint_prefix,
        "Service name `%s` does not match expected endpoint "
        "prefix `%s`, actual: `%s`" % (
            service_name, expected_endpoint_prefix, endpoint_prefix))



#!/usr/bin/env
# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import botocore.config
from tests import unittest
import mock

from botocore import args
from botocore.config import Config


class TestCreateClientArgs(unittest.TestCase):
    def setUp(self):
        self.args_create = args.ClientArgsCreator(None, None, None, None, None)

    def test_compute_s3_configuration(self):
        scoped_config = {}
        client_config = None
        self.assertIsNone(
            self.args_create.compute_s3_config(
                scoped_config, client_config))

    def test_compute_s3_config_only_scoped_config(self):
        scoped_config = {
            's3': {'use_accelerate_endpoint': True},
        }
        client_config = None
        self.assertEqual(
            self.args_create.compute_s3_config(scoped_config, client_config),
            {'use_accelerate_endpoint': True}
        )

    def test_client_s3_accelerate_from_varying_forms_of_true(self):
        scoped_config= {'s3': {'use_accelerate_endpoint': 'True'}}
        client_config = None

        self.assertEqual(
            self.args_create.compute_s3_config(
                {'s3': {'use_accelerate_endpoint': 'True'}},
                client_config=None),
            {'use_accelerate_endpoint': True}
        )
        self.assertEqual(
            self.args_create.compute_s3_config(
                {'s3': {'use_accelerate_endpoint': 'true'}},
                client_config=None),
            {'use_accelerate_endpoint': True}
        )
        self.assertEqual(
            self.args_create.compute_s3_config(
                {'s3': {'use_accelerate_endpoint': True}},
                client_config=None),
            {'use_accelerate_endpoint': True}
        )

    def test_client_s3_accelerate_from_client_config(self):
        self.assertEqual(
            self.args_create.compute_s3_config(
                scoped_config=None,
                client_config=Config(s3={'use_accelerate_endpoint': True})
            ),
            {'use_accelerate_endpoint': True}
        )

    def test_client_s3_accelerate_client_config_overrides_scoped(self):
        self.assertEqual(
            self.args_create.compute_s3_config(
                scoped_config={'s3': {'use_accelerate_endpoint': False}},
                client_config=Config(s3={'use_accelerate_endpoint': True})
            ),
            # client_config beats scoped_config
            {'use_accelerate_endpoint': True}
        )

    def test_client_s3_dualstack_handles_varying_forms_of_true(self):
        scoped_config= {'s3': {'use_dualstack_endpoint': 'True'}}
        client_config = None

        self.assertEqual(
            self.args_create.compute_s3_config(
                {'s3': {'use_dualstack_endpoint': 'True'}},
                client_config=None),
            {'use_dualstack_endpoint': True}
        )
        self.assertEqual(
            self.args_create.compute_s3_config(
                {'s3': {'use_dualstack_endpoint': 'true'}},
                client_config=None),
            {'use_dualstack_endpoint': True}
        )
        self.assertEqual(
            self.args_create.compute_s3_config(
                {'s3': {'use_dualstack_endpoint': True}},
                client_config=None),
            {'use_dualstack_endpoint': True}
        )

    def test_max_pool_from_client_config_forwarded_to_endpoint_creator(self):
        args_create = args.ClientArgsCreator(
            mock.Mock(), None, None, None, None)
        config = botocore.config.Config(max_pool_connections=20)
        service_model = mock.Mock()
        service_model.metadata = {
            'serviceFullName': 'MyService',
            'protocol': 'query'
        }
        service_model.operation_names = []
        bridge = mock.Mock()
        bridge.resolve.return_value = {
            'region_name': 'us-west-2', 'signature_version': 'v4',
            'endpoint_url': 'https://ec2/',
            'signing_name': 'ec2', 'signing_region': 'us-west-2',
            'metadata': {}}
        with mock.patch('botocore.args.EndpointCreator') as m:
            args_create.get_client_args(
                service_model, 'us-west-2', True, 'https://ec2/', True,
                None, {}, config, bridge)
            m.return_value.create_endpoint.assert_called_with(
                mock.ANY, endpoint_url='https://ec2/', region_name='us-west-2',
                response_parser_factory=None, timeout=(60, 60), verify=True,
                max_pool_connections=20, proxies=None
            )

    def test_proxies_from_client_config_forwarded_to_endpoint_creator(self):
        args_create = args.ClientArgsCreator(
            mock.Mock(), None, None, None, None)
        proxies = {'http': 'http://foo.bar:1234',
                   'https': 'https://foo.bar:4321'}
        config = botocore.config.Config(proxies=proxies)
        service_model = mock.Mock()
        service_model.metadata = {
            'serviceFullName': 'MyService',
            'protocol': 'query'
        }
        service_model.operation_names = []
        bridge = mock.Mock()
        bridge.resolve.return_value = {
            'region_name': 'us-west-2', 'signature_version': 'v4',
            'endpoint_url': 'https://ec2/',
            'signing_name': 'ec2', 'signing_region': 'us-west-2',
            'metadata': {}}
        with mock.patch('botocore.args.EndpointCreator') as m:
            args_create.get_client_args(
                service_model, 'us-west-2', True, 'https://ec2/', True,
                None, {}, config, bridge)
            m.return_value.create_endpoint.assert_called_with(
                mock.ANY, endpoint_url='https://ec2/', region_name='us-west-2',
                response_parser_factory=None, timeout=(60, 60), verify=True,
                proxies=proxies, max_pool_connections=10
            )

    def test_s3_with_endpoint_url_still_resolves_region(self):
        self.args_create = args.ClientArgsCreator(
            mock.Mock(), None, None, None, None)
        service_model = mock.Mock()
        service_model.endpoint_prefix = 's3'
        service_model.metadata = {'protocol': 'rest-xml'}
        config = botocore.config.Config()
        bridge = mock.Mock()
        bridge.resolve.side_effect = [
            {
                'region_name': None, 'signature_version': 's3v4',
                'endpoint_url': 'http://other.com/', 'signing_name': 's3',
                'signing_region': None, 'metadata': {}
            },
            {
                'region_name': 'us-west-2', 'signature_version': 's3v4',
                'enpoint_url': 'https://s3-us-west-2.amazonaws.com',
                'signing_name': 's3', 'signing_region': 'us-west-2',
                'metadata': {}
            }
        ]
        client_args = self.args_create.get_client_args(
            service_model, 'us-west-2', True, 'http://other.com/', True, None,
            {}, config, bridge)
        self.assertEqual(
            client_args['client_config'].region_name, 'us-west-2')

    def test_region_does_not_resolve_if_not_s3_and_endpoint_url_provided(self):
        self.args_create = args.ClientArgsCreator(
            mock.Mock(), None, None, None, None)
        service_model = mock.Mock()
        service_model.endpoint_prefix = 'ec2'
        service_model.metadata = {'protocol': 'query'}
        config = botocore.config.Config()
        bridge = mock.Mock()
        bridge.resolve.side_effect = [{
            'region_name': None, 'signature_version': 'v4',
            'endpoint_url': 'http://other.com/', 'signing_name': 'ec2',
            'signing_region': None, 'metadata': {}
        }]
        client_args = self.args_create.get_client_args(
            service_model, 'us-west-2', True, 'http://other.com/', True, None,
            {}, config, bridge)
        self.assertEqual(client_args['client_config'].region_name, None)

    def test_provide_retry_config(self):
        self.args_create = args.ClientArgsCreator(
            mock.Mock(), None, None, None, None)
        service_model = mock.Mock()
        service_model.endpoint_prefix = 'ec2'
        service_model.metadata = {'protocol': 'query'}
        config = botocore.config.Config(
            retries={'max_attempts': 10}
        )
        bridge = mock.Mock()
        bridge.resolve.side_effect = [{
            'region_name': None, 'signature_version': 'v4',
            'endpoint_url': 'http://other.com/', 'signing_name': 'ec2',
            'signing_region': None, 'metadata': {}
        }]
        client_args = self.args_create.get_client_args(
            service_model, 'us-west-2', True, 'https://ec2/', True, None,
            {}, config, bridge)
        self.assertEqual(
            client_args['client_config'].retries, {'max_attempts': 10})
